MILLI_MICROS = 10**3
SECOND_MICROS = MILLI_MICROS * 10**3
MINUTE_MICROS = SECOND_MICROS * 60
HOUR_MICROS = MINUTE_MICROS * 60
MEAN_DAY_MICROS = HOUR_MICROS * 24
MEAN_WEEK_MICROS = MEAN_DAY_MICROS * 7
MEAN_MONTH_MICROS = (MEAN_DAY_MICROS * (365*4+1)) / (12*4)
MEAN_YEAR_MICROS = (MEAN_DAY_MICROS * (365*4+1)) / 4

HALF_MILLI_MICROS = MILLI_MICROS / 2
HALF_SECOND_MICROS = SECOND_MICROS / 2
HALF_MINUTE_MICROS = MINUTE_MICROS / 2
HALF_HOUR_MICROS = HOUR_MICROS / 2
HALF_MEAN_DAY_MICROS = MEAN_DAY_MICROS / 2
HALF_MEAN_WEEK_MICROS = MEAN_WEEK_MICROS / 2
HALF_MEAN_MONTH_MICROS = MEAN_MONTH_MICROS / 2
HALF_MEAN_YEAR_MICROS = MEAN_YEAR_MICROS / 2

